<?php
/**
 * @version		$Id: lib.php,v 1.57 2009/09/10 17:26:03 juanca Exp $
 * @package		vpl. functions to coordinate with moodle
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

define('VPL','vpl');
define('VPL_SUBMISSIONS','vpl_submissions');
define('VPL_GRADE_CAPABILITY','mod/vpl:grade');
define('VPL_VIEW_CAPABILITY','mod/vpl:view');
define('VPL_SUBMIT_CAPABILITY','mod/vpl:submit');
define('VPL_SIMILARITY_CAPABILITY','mod/vpl:similarity');
define('VPL_MANAGE_CAPABILITY','mod/vpl:manage');

require_once('vpl.class.php');
/**
 * Open/create a file and its dir
 *
 * @param $filename path to file
 * @return file descriptor
 **/
function vpl_fopen($filename){
	if(!file_exists($filename)){ //¿exist file?
		$dir = dirname($filename);
		if(!file_exists($dir)){ //¿create dir?
			mkdir($dir, 0700,true);
		}
	}
	$fp= fopen($filename,'wb+');
	return $fp;
}

/**
 * Recursive delete a directory
 *
 * @return bool All delete
 **/
function vpl_delete_dir($dirname){
	$ret = false;
	if(file_exists($dirname)){
		$ret = true;
		if(is_dir($dirname)){
			$dd = opendir($dirname);
			if(!$dd){
				return false;
			}
			$list = array();
			while($name=readdir($dd)){
				if($name != '.' && $name != '..'){
					$list[]=$name;
				}
			}
			closedir($dd);
			$ret = true;
			foreach ($list as $name) {
				$ret = $ret and vpl_delete_dir($dirname.'/'.$name);
			}
			$ret = $ret and rmdir($dirname);
		}
		else{
			$ret = unlink($dirname);
		}
	}
	return $ret;
}

/**
 * get parsed lines of a file
 *
 * @param $filename string
 * @return array of lineasfiles required name
 **/
function vpl_read_list_from_file($filename){
	$ret = array();
	if(file_exists($filename)){
		$data = file_get_contents($filename);
		$nl = vpl_detect_newline($data);
		$ret = explode($nl,$data);
	}
	return $ret;
}

/**
 * Save an array in a file
 * @param $filename string
 **/
function vpl_write_list_to_file($filename, $list){
	$data='';
	foreach ($list as $info) {
		if($info > ''){
			if($data > ''){
				$data .= "\n";
			}
			$data .= $info;
		}
	}
	$fp = vpl_fopen($filename);
	fwrite($fp,$data);
	fclose($fp);
}

/**
 * Get lang code
 * @parm $bash_adapt true adapt lang to bash LANG (default false)
 * @return string
 */
function vpl_get_lang($bash_adapt=false){
	global $SESSION,$USER,$CFG;
	if(isset($SESSION->lang)){
		$lang=$SESSION->lang;
	}elseif(isset($USER->lang)){
		$lang=$USER->lang;
	}else{
		$lang=$CFG->lang;
	}
	if($bash_adapt){
		$parts=explode('_',$lang);
		switch(count($parts)){
			case 2:
				if($parts[1]=='utf8'){
					$lang=$parts[0].'.UTF-8';
				}else{
					$lang=$parts[0].'_'.strtoupper($parts[1]);
				}
				break;
			case 3:
				if($parts[2]=='utf8'){
					$lang=$parts[0].'_'.strtoupper($parts[1]).'.UTF-8';
				}else{
					$lang=$parts[0].'_'.strtoupper($parts[1]).'.'.$parts[2];
				}
				break;
			default: //Do nothing
		}
	}
	return $lang;
}

/**
 * generate URL to page with params
 * @param $page string page from wwwroot
 * @param $var1 string var1 name optional
 * @param $value1 string value of var1 optional
 * @param $var2 string var2 name optional
 * @param $value2 string value of var2 optional
 * @param ...
 **/
function vpl_abs_href(){
	global $CFG;
	$parms = func_get_args();
	$l=count($parms);
	$href = $CFG->wwwroot.$parms[0];
	for( $p=1;$p<$l-1; $p+=2 ) {
		if($parms[$p] && $parms[$p+1]){
			$href .= ($p>1?'&amp;':'?')
				.urlencode($parms[$p]).'='
				.urlencode($parms[$p+1]);
		}
	}
	return $href;
}

/**
 * generate URL to page with params
 * @param $page string page from wwwroot/mod/vpl/
 * @param $var1 string var1 name optional
 * @param $value1 string value of var1 optional
 * @param $var2 string var2 name optional
 * @param $value2 string value of var2 optional
 * @param ...
 **/
function vpl_mod_href(){
	global $CFG;
	$parms = func_get_args();
	$l=count($parms);
	$href = $CFG->wwwroot.'/mod/vpl/'.$parms[0];
	for( $p=1;$p<$l-1; $p+=2 ) {
		if($parms[$p] && $parms[$p+1]){
			$href .= ($p>1?'&amp;':'?')
				.urlencode($parms[$p]).'='
				.urlencode($parms[$p+1]);
		}
	}
	return $href;
}

/**
 * generate URL relative page with params
 * @param $page string page relative
 * @param $var1 string var1 name optional
 * @param $value1 string value of var1 optional
 * @param $var2 string var2 name optional
 * @param $value2 string value of var2 optional
 * @param ...
 **/
function vpl_rel_url(){
	$parms = func_get_args();
	$l=count($parms);
	$url = $parms[0];
	for( $p=1;$p<$l-1; $p+=2 ) {
		if($parms[$p] && $parms[$p+1]){
			$url .= ($p>1?'&amp;':'?')
				.urlencode($parms[$p]).'='
				.urlencode($parms[$p+1]);
		}
	}
	return $url;
}
/**
 * Add a parm to a url
 * @param $url string
 * @param $parm string name
 * @param $value string value of parm
 **/
function vpl_url_add_param($url,$parm,$value){
	if(strpos($url,'?')){
		return $url . '&amp;'.urlencode($parm).'='.urlencode($value);
	}else{
		return $url . '?'.urlencode($parm).'='.urlencode($value);
	}
}

function vpl_include_jsfile($file,$defer=false){
	$ret='<script type="text/javascript"';
	if($defer){
		$ret.=' defer="defer"';
	}
	$url = vpl_mod_href('jscript/'.$file);
	$ret .=' src="'.$url.'"></script>';
	return $ret;
}

function vpl_include_js($jscript,$defer=false){
	$ret='<script type="text/javascript"';
	if($defer){
		$ret.=' defer="defer">';
	}else{
		$ret.='>';
	}
	$ret .= $jscript.'</script>';
	return $ret;
}

function vpl_js_alert($text,$print=true){
	$ret=vpl_include_js('alert("'.s($text).'")');
	if($print){
		echo $ret;
		@ob_flush();
		flush();
	}else{
		return $ret;
	}
}

function vpl_get_select_time($limit=null){
	$minute=60;
	if($limit == null){
		$limit = 35*$minute;
	}
	$ret = array(0 => get_string('select'));
	$value=1;
	if($limit < $value){
		$value=$limit;
	}
	while($value <= $limit){
		if($value < $minute){
			$ret[$value] = "$value".' '.get_string('seconds',VPL);
		}else{
			$num = (int)($value / $minute);
			if($num==1){
				$ret[$num*$minute] = "$num".' '.get_string('minute',VPL);
			}else{
				$ret[$num*$minute] = "$num".' '.get_string('minutes',VPL);
			}
			$value = $num*$minute;
		}
		$value *=2;
	}
	return $ret;
}

/**
 * Return the post_max_size PHP config option in bytes
 * @return int max size in bytes
 */
function vpl_get_max_post_size(){
    $maxs = trim(ini_get('post_max_size'));
    $len = strlen($maxs);
    $last = strtolower($maxs[$len-1]);
    $max = (int) substr($maxs,0,$len-1);
    if($last == 'k'){
    	$max *= 1024;
    }elseif ($last == 'm'){
    	$max *= 1024*1024;
    }elseif ($last == 'g'){
    	$max *= 1024*1024*1000;
    }
    return $max;
}

/**
 * Return un array with the format [size in bytes]=> size in text text
 * The first element is [0] => select 
 * @param $limit the limit of values generates
 * @return array
 */
function vpl_get_select_sizes($limit=null){
	$Kbyte=1024;
	$Mbyte=1000*$Kbyte;
	$Gbyte=1000*$Mbyte;
	$Tbyte=1000*$Gbyte;
	if($limit == null){
		$limit = 1024*1024*1024;
	}
	$ret = array(0 => get_string('select'));
	$value=80*1024;
	if($limit < $value){
		$limit=$value;
	}
	while($value <= $limit){
		if($value < $Mbyte){
			$num = $value / $Kbyte;
			$ret[$value] = "$num".'KB';
		}else if($value < $Gbyte){
			$num = (int)($value / $Mbyte);
			$ret[$value] = "$num".'MB';
			$value = $num*$Mbyte;
		}else if($value < $Tbyte){
			$num = $value / $Gbyte;
			$ret[$value] = "$num".'GB';
		}else{
			$num = $value / $Tbyte;
			$ret[$value] = "$num".'TB';
		}
		$value *=2;
	}
	return $ret;
}

/**
 * @param string $data
 * @return string newline separator "\r\n", "\n\r", "\n"
 */
function vpl_detect_newline(&$data){
	//Detect newline type
	if(strrpos($data,"\r\n")){
		return "\r\n"; //Windows
	}else if(strrpos($data,"\n")){
		return "\n"; //UNIX
	}
	return "\r"; //Mac
}

function vpl_display_grade($grade,$scaleid) {
	global $CFG;
	require_once $CFG->libdir.'/gradelib.php';
	static $scalegrades = array();   //Cache scales
	if ($scaleid >= 0) {    //Normal scale
		if ($grade == -1) {
			return '-';
		} else {
			return $grade.' / '.$scaleid;
		}
	} else {
		$scaleid = -$scaleid;
		if (empty($scalegrades[$scaleid])) {
			if ($scale = get_record('scale', 'id', $scaleid)) {
				$scalegrades[$scaleid] = make_menu_from_list($scale->scale);
			} else {
				return '-';
			}
		}
		if (isset($scalegrades[$scaleid][$grade])) {
			return $scalegrades[$scaleid][$grade];
		}
		return '-';
	}
}

/**
 * Create grade item for vpl
 *
 * @param object $assignment object with extra cmidnumber
 * @param mixed optional array/object of grade(s); 'reset' means reset grades in gradebook
 * @return int 0 if ok, error code otherwise
 */
function vpl_update_grade_item($instance,$cm=null) {
	$itemdetails = array('itemname' => $instance->name);
	if($cm){
		$itemdetails['idnumber']= $cm->id;
	}
	if ($instance->grade > 0) {
		$itemdetails['gradetype'] = GRADE_TYPE_VALUE;
		$itemdetails['grademax']  = $instance->grade;
		$itemdetails['grademin']  = 0;
	} else if ($instance->grade < 0) {
		$itemdetails['gradetype'] = GRADE_TYPE_SCALE;
		$itemdetails['scaleid']   = -$instance->grade;
	} else {
		$itemdetails = array('deleted'=>1);
	}
	$itemdetails['hidden'] = ($instance->visiblegrade)?0:1;
	grade_update('mod/vpl', $instance->course, 'mod', VPL,
	$instance->id, 0, NULL, $itemdetails);
}

/**
 * Delete grade_item from a vpl instance+id
 *
 * @param $instance vpl instance
 **/
function vpl_delete_grade_item($instance) {
	global $CFG;
	require_once $CFG->libdir.'/gradelib.php';
	$itemdetails = array('deleted'=>1);
	grade_update('mod/vpl', $instance->course, 'mod', VPL,$instance->id, 0, NULL, $itemdetails);
}


/**
 * Create an event object from a vpl instance+id
 *
 * @param $instance vpl instance
 * @param $id vpl instance id
 * @return event object
 **/
function vpl_create_event($instance,$id){
	$event = new object();
	$event->name        = $instance->name;
	$event->description = $instance->shortdescription;
	$event->courseid    = $instance->course;
	$event->modulename  = VPL;
	$event->instance    = $id;
	$event->eventtype   = 'due';
	$event->timestart   = $instance->duedate;
	return $event;
}

/**
 * Add a new vpl instance and return the id
 *
 * @param object from the form in mod_form.html
 * @return int id of the new vpl
 **/
function vpl_add_instance($instance) {
	$id = insert_record(VPL, $instance);
	//Add event
	if ($instance->duedate) {
		add_event(vpl_create_event($instance,$id));
	}
	//Add grade to grade book
	$instance->id=$id;
	vpl_update_grade_item($instance);
	return $id;
}

/**
 * Update a vpl instance
 *
 * @param object from the form in mod.html
 * @return boolean OK
 *
 **/
function vpl_update_instance($instance) {
	$instance->id = $instance->instance;
	//Update event
	if ($instance->duedate) {
		$event = vpl_create_event($instance,$instance->id);
		if ($eventid = get_field('event', 'id', 'modulename', VPL, 'instance', $instance->id)) {
			$event->id = $eventid;
			update_event($event);
		} else {
			add_event($event);
		}
	} else {
		delete_records('event', 'modulename', VPL, 'instance', $instance->id);
	}
	$cm = get_coursemodule_from_instance(VPL, $instance->id, $instance->course);
	vpl_update_grade_item($instance,$cm);
	return update_record(VPL, $instance);
}

/**
 * Delete an instance by id
 *
 * @param int $id Id instance
 * @return boolean OK
 **/
function vpl_delete_instance($id) {
	$vpl = new mod_vpl(false,$id);
	$res = $vpl->delete_all();
	//Locate related VPLs and reset its basedon $id to 0
	$related = get_records_select(VPL,"basedon = $id",'id','id,basedon');
	if($related !== false){
		foreach($related as $other){
			$other->basedon=0;
			update_record(VPL,$other);
		}
	}
	return $res;
}

/**
 * Return an object with short information about what a
 * user has done with a given particular instance of this module
 * $return->time = the time they did it
 * $return->info = a short text description
 * @param $course
 * @param $user
 * @param $mod
 * @param $instance
 * @return the object info
 **/
function vpl_user_outline($course, $user, $mod, $instance) {
	//Search submisions for $user $instance
	$vpl = new mod_vpl(null,$instance->id);
	$subinstance = $vpl->last_user_submission($user->id);
	if(!$subinstance) {
		$return = null;
	}
	else{
		require_once('vpl_submission.class.php');
		$return = new stdClass;
		$submission = new mod_vpl_submission($vpl,$subinstance);
		$return->time= $subinstance->datesubmitted;
		$subs=$vpl->user_submissions($user->id);
		if(count($subs)>1){
			$info = get_string('submissions',VPL,count($subs));
		}else{
			$info = get_string('submission',VPL,count($subs));
		}
		if($subinstance->dategraded){
			$info .='<br />'.get_string('grade').': '.$submission->print_grade_core();
		}
		$url=vpl_mod_href('forms/submissionview.php','id',$vpl->get_course_module()->id,'userid',$user->id);
		$return->info = '<a href="'.$url.'">'.$info.'</a>';
	}
	return $return;
}

/**
 * Print a detailed report of what a user has done
 * with a given particular instance of this module
 * @param $course
 * @param $user
 * @param $mod
 * @param $instance
 * @return boolean OK
 **/
function vpl_user_complete($course, $user, $mod, $vpl) {
	require_once('vpl_submission.class.php');
	//TODO Print a detailed report of what a user has done with a given particular instance
	//Search submisions for $user $instance
	$vpl = new mod_vpl(null,$vpl->id);
	$sub = $vpl->last_user_submission($user->id);
	if(!$sub) {
		$return = null;
	}
	else{
		$submission = new mod_vpl_submission($vpl,$sub);
		$submission->print_info(true);
		$submission->print_grade(true);
	}
	return true;
}


/**
 * Print activity of a course since a time
 *
 * @uses $CFG
 * @param $course course id
 * @param $isteacher
 * @param $timestart activity from
 * @return boolean (true==something printed)
 **/
function vpl_print_recent_activity($course, $isteacher, $timestart) {
	//TODO FIXME
	return false;
    global $CFG;
    global $USER;
    $select ="(datesubmitted >= $timestart) or (dategraded >=$timestart)";
    $subs = get_records_select(VPL_SUBMISSIONS,$select,'datesubmitted DESC');
    if (!$subs){
        return false;
    }

    $modinfo = get_fast_modinfo($course);
    $activity = array();
	//$context = array();
	$first = true;
    foreach ($subs as $sub) {
        $cm = $modinfo->instances[VPL][$sub->vpl];
        if(!isset($context[$cm->id])){ //Cache context
        	$context[$cm->id] = get_context_instance(CONTEXT_MODULE,$cm->id);
        }
        $context = get_context_instance(CONTEXT_MODULE,$context[$cm->id]);
        //Show if own user & visible or VPL_GRADE_CAPABILITY
        if(($USER->id == $sub->userid &&  $cm->uservisible)||
           has_capability(VPL_GRADE_CAPABILITY,$context) ){
           	if($first){
           		$fisrt=false;
    			print_headline(get_string('submision', VPL), 3);
           	}
           	$url=vpl_abs_href_url('forms/submissionview.php','id',$id,
           						'userid',$sub->userid,'submissionid',$sub->id);
           	//Submited in period
           	if($sub->datesubmitted>$timestart){
        		print_recent_activity_note($sub->datesubmitted,
        									$sub->userid,
        									get_string('submission',VPL),
     										$url);
           	}
           	//Grade in period
           	if($sub->dategraded>$timestart){
        		print_recent_activity_note($sub->dategraded,
        									$sub->graderid,
        									get_string('grade',VPL),
     										$url);
           	}
        }
    }
    return !$first;
}


/**
 * Run periodically to check for vpl visibility update
 *
 * @uses $CFG
 * @return boolean
 **/
function vpl_cron() {
	global $CFG;
	$now = time();
	$vpls = get_records_select(VPL,'','','id,visiblefrom');
	foreach ($vpls as $instance) {
		if($instance->visiblefrom>0 && $now >$instance->visiblefrom){ //Module instance will be show from time
			$vpl = new mod_vpl(null,$instance->id);
			//TODO Test
			if(!instance_is_visible(VPL,$vpl->get_instance())){
				set_coursemodule_visible($vpl->get_course_module()->id,true,true);
			}
		}
	}
	return true;
}

/**
 * Must return an array of user records (all data) who are participants
 * for a given instance of vpl. Must include every user involved
 * in the instance, independient of his role (student, teacher, admin...)
 * See other modules as example.
 *
 * @param int $vplid ID of an instance of this module
 * @return mixed boolean/array of users
 **/
function vpl_get_participants($vplid) {
	global $CFG;
	//Locate students
	$submiters = get_records_sql("SELECT DISTINCT userid
	FROM {$CFG->prefix}vpl_submissions
	WHERE vpl = $vplid");
	//Locate graders
	$graders = get_records_sql("SELECT DISTINCT grader
	FROM {$CFG->prefix}vpl_submissions
	WHERE vpl = $vplid");

	//Read users records
	$participants = array();
	if($submiters){
		foreach ($submiters as $submiter) {
			$user = get_record('user','id',$submiter->userid);
			if($user){
				$participants[$user->id] = $user;
			}
		}
	}
	if($graders){
		foreach ($graders as $grader) {
			$user = get_record('user','id',$grader->grader);
			if($user){
				$participants[$user->id] = $user;
			}
		}
	}
	if(count($participants)>0){
		return $participants;
	}else{
		return false;
	}
}

function vpl_scale_used ($vplid,$scaleid) {
	return $scaleid and record_exists(VPL, 'id',"$vplid",'grade', "-$scaleid");
}


/**
 * Checks if scale is being used by any instance of vpl.
 * This is used to find out if scale used anywhere
 * @param $scaleid int
 * @return boolean True if the scale is used by any vpl
 */
function vpl_scale_used_anywhere($scaleid) {
	return $scaleid and record_exists(VPL, 'grade', "-$scaleid");
}

/**
 * Execute post-install custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function vpl_install() {
	return true;
}

/**
 * Execute post-uninstall custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function vpl_uninstall() {
	return true;
}

/*
 * any/all functions defined by the module should be in here. If the modulename is called widget, then the required functions include:
 * Other functions available but not required are:
 o widget_delete_course() - code to clean up anything that would be leftover after all instances are deleted
 o widget_process_options() - code to pre-process the form data from module settings
 o widget_reset_course_form() and widget_delete_userdata() - used to implement Reset course feature.
 * To avoid possible conflict, any module functions should be named starting with widget_ and any constants you define should start with WIDGET_
 */

?>