<?php
/**
 * @version		$Id: vpl_submission.class.php,v 1.82 2009/08/18 12:24:56 juanca Exp $
 * @package		vpl. submission class definition
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

/**
 * Module instance files
 * root_path= vpl_data/course_id#/vpl_instance#
 * Submission info
 * root_path/usersdata/userid#/submissionid#/submissionfilelist.txt
 * root_path/usersdata/userid#/submissionid#/submissionfiles/
 * root_path/usersdata/userid#/submissionid#/grade_comments.txt
 * root_path/usersdata/userid#/submissionid#/teachertest.txt
 * root_path/usersdata/userid#/submissionid#/studenttest.txt
 */

require_once('vpl.class.php');
require_once('views/sh_factory.class.php');
class mod_vpl_submission {
	protected $vpl;
	protected $instance;

	/**
	 * Internal var object to submitted file group manager
	 * @var object of file group manager
	 */
	protected $submitted_fgm;

	/**
	 * Constructor
	 * @param $vpl object vpl instance
	 * @param $instance vpl submission
	 **/
	function __construct($vpl, $mix) {
		$this->vpl = $vpl;
		if(is_object($mix)){
			$this->instance = $mix;
		}
		else{
			$this->instance = get_record('vpl_submissions','id',$mix);
			if(!$this->instance){
				print_error('vpl_submission not found');
			}
		}
		$this->submitted_fgm = null;
	}

	/**
	 * get submission instance
	 * @return object submission instance
	 **/
	function get_instance(){
		return $this->instance;
	}

	/**
	 * get path to data submission directory
	 * @return string submission data directory
	 **/
	function get_data_directory(){
		return $this->vpl->get_users_data_directory().'/'.
		$this->instance->userid.'/'.$this->instance->id.'/';
	}

	/**
	 * get path to files submission directory
	 * @return string files submission directory
	 **/
	function get_submission_directory(){
		return $this->get_data_directory().'submitedfiles/';
	}

	/**
	 * get absolute path to name of file with list of submitted files
	 * @return string file name
	 **/
	function get_submissionfilelistname(){
		return $this->get_data_directory().'submitedfilelist.txt';
	}

	/**
	 * @return object file group manager for submitted files
	 **/
	function get_submitted_fgm(){
		if(!$this->submitted_fgm){
			$this->submitted_fgm = new file_group_process($this->get_submissionfilelistname(), $this->get_submission_directory());
		}
		return $this->submitted_fgm;
	}

	/**
	 * get absolute path to the grade comments file name
	 * @return string file name
	 **/
	function get_gradecommentsfilename(){
		return $this->get_data_directory().'grade_comments.txt';
	}

	/**
	 * get submitted files
	 * @return array of array 'name' and 'data'
	 **/
	function get_submited_files(){
		//TODO refactor to file_group_process
		$ret = array();
		$filelist = vpl_read_list_from_file($this->get_submissionfilelistname());
		$basedir = $this->get_submission_directory();
		foreach ($filelist as $filename) {
			if($filename > ''){
				$data = file_get_contents($basedir.$filename);
				$ret[] = array('name' => $filename, 'data' => $data);
			}
		}
		return $ret;
	}

	function set_submited_file($files){
		//TODO refactor to file_group_process
		$filelist=array();
		$basepath = $this->get_submission_directory();
		foreach($files as $file){
			$filename = basename($file['name']);
			if($filename>''){
				$filelist[] = $filename;
				$fp = vpl_fopen($basepath.$filename);
				fwrite($fp,$file['data']);
				fclose($fp);
			}
		}
		vpl_write_list_to_file($this->get_submissionfilelistname(),$filelist);
	}

	/**
	 * Delete submited files and own record
	 * @return void
	 **/
	function delete(){
		vpl_delete_dir($this->get_data_directory());
		delete_records('vpl_submissions','id',$this->instance->id);
	}
	
	/**
	 * Return if submission has been graded
	 * @return bool
	 */
	function is_graded(){
		return $this->instance->dategraded>0;
	}

	/**
	 * Remove grade
	 *
	 */
	function remove_grade(){
		global $USER;
		global $CFG;
		if (!function_exists('grade_update')) {
			require_once($CFG->libdir.'/gradelib.php');
		}
		$this->instance->grader=0;
		$this->instance->dategraded=0;
		$this->instance->grade=null;
		$fn = $this->get_gradecommentsfilename();
		if(file_exists($fn)){
			unlink($fn);
		}
		$grades= array();
		$gradeinfo= array();
		$gradeinfo['userid'] =$this->instance->userid;
		$gradeinfo['delete'] = 1;
		$grades[$this->instance->userid] = $gradeinfo;
		grade_update('mod/vpl',$this->vpl->get_course()->id,'mod',VPL,
		$this->vpl->get_instance()->id,0,$grades);
		if(!empty($CFG->enableoutcomes)){
			$grading_info = grade_get_grades($this->vpl->get_course()->id, 'mod', 'vpl',
			$this->vpl->get_instance()->id, $this->instance->userid);
			if (!empty($grading_info->outcomes)) {
				$userid=$this->instance->userid;
				$outcomes = array();
				foreach($grading_info->outcomes as $oid=>$dummy) {
					$outcomes[$oid]=null;
				}
				grade_update_outcomes('mod/vpl',$this->vpl->get_course()->id,'mod',VPL,
					$this->vpl->get_instance()->id,$userid,$outcomes);
			}
		}
		$instance_slashed=addslashes_object($this->instance);
		if(!update_record('vpl_submissions',$instance_slashed)){
			print_error('DB error updating submission grade info');
		}
	}
	
	/**
	 * Set/update grade
	 *
	 */
	function set_grade($info){
		global $USER;
		global $CFG;
		if($this->vpl->get_instance()->grade==0 &&
		   empty($CFG->enableoutcomes)){
		   	return;
		}
		if (!function_exists('grade_update')) {
			require_once($CFG->libdir.'/gradelib.php');
		}
		$this->instance->grader=$USER->id;
		$this->instance->dategraded=time();
		if($this->vpl->get_instance()->grade!=0){
			$this->instance->grade=$info->grade;
			$comments=$info->comments;
			$fn = $this->get_gradecommentsfilename();
			if($comments){
				$fp = vpl_fopen($fn);
				//Write comments with slashes removed
				fwrite($fp,stripslashes_safe($comments));
				fclose($fp);
			} elseif(file_exists($fn)){
				unlink($fn);
			}
			$grades= array();
			$gradeinfo= array();
			$gradeinfo['userid'] =$this->instance->userid;
			$gradeinfo['rawgrade'] = $info->grade;

			if($comments && $comments>''){
				$gradeinfo['feedback'] = $this->result_to_HTML($info->comments);
				$gradeinfo['feedbackformat'] = FORMAT_HTML;
			}
			$gradeinfo['usermodified'] = $USER->id;;
			$gradeinfo['datesubmitted'] = $this->instance->datesubmitted;
			$gradeinfo['dategraded'] = $this->instance->dategraded;
			$grades[$this->instance->userid] = $gradeinfo;
			grade_update('mod/vpl',$this->vpl->get_course()->id,'mod',VPL,
			$this->vpl->get_instance()->id,0,$grades);
		}
		if(!empty($CFG->enableoutcomes)){
			$grading_info = grade_get_grades($this->vpl->get_course()->id, 'mod', 'vpl',
			$this->vpl->get_instance()->id, $this->instance->userid);
			if (!empty($grading_info->outcomes)) {
				$userid=$this->instance->userid;
				$outcomes = array();
				foreach($grading_info->outcomes as $oid=>$dummy) {
					$field='outcome_grade_'.$oid;
					if(isset($info->$field)){
						$outcomes[$oid]=$info->$field;
					}else{
						$outcomes[$oid]=null;
					}
				}
				grade_update_outcomes('mod/vpl',$this->vpl->get_course()->id,'mod',VPL,
					$this->vpl->get_instance()->id,$userid,$outcomes);
			}
		}
		$instance_slashed=addslashes_object($this->instance);
		if(!update_record('vpl_submissions',$instance_slashed)){
			print_error('DB error updating submission grade info');
		}
	}

	/**
	 * Get grade comments
	 *
	 * @return string
	 */
	function get_grade_comments(){
		$fn = $this->get_gradecommentsfilename();
		if(file_exists($fn)){
			return file_get_contents($fn);
		}
		else{
			return null;
		}
	}

	/**
	 * is visible this submission instance
	 * @return bool
	 */
	function is_visible(){
		global $USER;
		$cm = $this->vpl->get_course_module();
		$instance = $this->instance;
		$ret = $this->vpl->is_visible();
		//submission owner?
		$ret = $ret && ($USER->id == $instance->userid);
		if($ret){
			//is last submission?
			$lastsub = $this->vpl->last_user_submission($instance->userid);
			$ret = $ret && ($instance->id == $lastsub->id);
		}
		$ret = $ret || $this->vpl->has_capability(VPL_GRADE_CAPABILITY);
		return $ret;
	}

	/**
	 * is possible to grade/update grade this submission instance
	 * @return bool
	 */
	function is_grade_able(){
		global $USER;
		if($this->vpl->get_instance()->grade==0){//Is grade_able the instance
			return false;
		}
		$instance = $this->instance;
		$ret = $this->vpl->has_capability(VPL_GRADE_CAPABILITY);;
		//new grade or update if grader
		$ret = $ret && ($instance->dategraded || $USER->id == $instance->grader);
		$ret = $ret || $this->vpl->has_capability(VPL_MANAGE_CAPABILITY);
		if($ret){
			//is last submission?
			$lastsub = $this->vpl->last_user_submission($instance->userid);
			$ret = $ret && ($instance->id == $lastsub->id);
		}
		return $ret;
	}

	/**
	 * Print sudmitted file
	 *
	 */
	function print_submitted_file($name,$data){
		$shower= vpl_sh_factory::get_sh($name);
		$shower->print_file($name,$data);
	}

	/**
	 * Print core grade
	 * @parm optional grade to show
	 * @return string
	 */
	function print_grade_core($grade=null){
		$ret = '';
		$inst=$this->instance;
	    if($inst->dategraded>0 || $grade != null){
			$vplinstance = $this->vpl->get_instance();
			$scaleid = $vplinstance->grade;
			$options = array();
			if($scaleid == 0){
				$ret = get_string('nograde');
			}
			if ($scaleid > 0) {
				if($grade == null){
					$grade=rtrim(rtrim($inst->grade,'0'),'.,');
				}
				$ret = $grade.' / '.$scaleid;
			} elseif($scaleid < 0) {
				$scaleid = -$scaleid;
				if($grade == null){
					$grade=trim($inst->grade);
				}
				$grade = (int)$grade;
				if ($scale = get_record('scale', 'id', $scaleid)) {
					//TODO nograde
					$options=array();
					$options[-1]= get_string('nograde');
					$options = $options + make_menu_from_list($scale->scale);
					if(isset($options[$grade])){
						$ret = $options[$grade];
					}
				}
			}
	    }
    	return $ret;
	}
	
	/**
	 * Print sudmission grade
	 * @param $detailed true show detailed grade (default false)
	 * @param $return true return string/ false print grade (default false)
	 * @return mix string/void
	 */
	function print_grade($detailed=false,$return=false){
		global $CFG;
		$ret = '';
		$inst=$this->instance;
	    if($inst->dategraded>0){
	    	$grader = get_record('user','id',$inst->grader);
	    	$a = new stdClass;
	    	$a->date=userdate($inst->dategraded);
	    	$a->gradername=fullname($grader);
	    	$ret .= get_string('gradedonby',VPL,$a).'<br />';
	    	if($this->vpl->get_instance()->grade!=0){
		    	$ret .= '<b>'.get_string('grade').'</b> '.$this->print_grade_core().'<br />';
		    	if($detailed){
		    		$feedback = $this->get_grade_comments();
		    		if($feedback){
		    			$ret .= '<b>'.get_string('gradercomments',VPL).'</b><br />';
		    			$ret .= $this->result_to_HTML($feedback);
		    		}
		    	}
	    	}
	    	if(!empty($CFG->enableoutcomes)){
				$grading_info = grade_get_grades($this->vpl->get_course()->id, 'mod', 'vpl',
				$this->vpl->get_instance()->id, $this->instance->userid);
				if (!empty($grading_info->outcomes)) {
					$ret .= '<b>'.get_string('outcomes','grades').'</b><br />';
					foreach($grading_info->outcomes as $oid=>$outcome) {
						$ret .= s($outcome->name);
						$ret .= ' '.s($outcome->grades[$inst->userid]->str_grade).'<br />';
					}
				}
			}
	    	
	    }
	    if($return){
	    	return $ret;
	    }else{
	    	if($ret){
	    		print_box($ret);
	    	}
	    }
	}
	
	/**
	 * Print sudmission info
	 *
	 */
	function print_info($autolink=false){
		//TODO improve show submission info
		$id=$this->vpl->get_course_module()->id;
		$userid=$this->instance->userid;
		$submissionid=$this->instance->id;
		if($autolink){
			$url=vpl_mod_href('forms/submissionview.php','id',$id,
						'userid',$userid,'submissionid',$submissionid);
			echo '<a href="'.$url.'">';		
		}
		$subdate=userdate($this->instance->datesubmitted);
	    p(get_string('submittedonp',VPL,$subdate));
		if($autolink){
			echo '</a>';	
		}
		$url=vpl_mod_href('views/downloadsubmission.php','id',$id,
						'userid',$userid,'submissionid',$submissionid);
		echo ' (<a href="'.$url.'">'.get_string('download',VPL);		
		echo '</a>)<br />';
		$commmets = $this->instance->comments;
		if($commmets){
			echo '<br />';
			echo get_string('comments',VPL);
			print_box($commmets);
		}
	}

	/**
	 * Print compilation and execution
	 * @return void
	 */
	function print_CE(){
		$ce = $this->getCE();
		if($ce === false){
			return;
		}
		if($ce['executed']){
			$raw_execution = $ce['execution'];
			$proposed_comments = $this->proposedComment($raw_execution);
			$proposed_grade = $this->proposedGrade($raw_execution);
			$comments=$this->result_to_HTML($proposed_comments);
			if($proposed_grade){
				print_box('<h3>'.get_string('proposedgrade',VPL,$proposed_grade).'</h3>');
			}
		}
		if($ce['compilation']){
			$compilation=$this->result_to_HTML($ce['compilation']);
			if($compilation){
				print_box_start();
				echo '<h3>'.get_string('compilation',VPL).'</h3>';
				echo $compilation;
				print_box_end();
			}
		}
		if($ce['executed']){
			if($comments){
				print_box_start();
				echo '<h3>'.get_string('comments',VPL).'</h3>';
				echo $comments;
				print_box_end();
			}
		}
	}
	/**
	 * Print sudmission
	 */
	function print_submission(){
		$this->print_info();
		$this->print_CE();
		$files = $this->get_submited_files();
		foreach ($files as $file) {
			$this->print_submitted_file($file['name'],$file['data']);
		}
	}

	const gradeTag='Grade :=>>';
	const commentTag='Comment :=>>';
	const beginCommentTag='<|--';
	const endCommentTag='--|>';

	function proposedGrade($text) {
		$ret = '';
		$nl = vpl_detect_newline($text);
		$lines = explode($nl,$text);
		foreach($lines as $line){
			if(strpos($line,self::gradeTag) === 0){
				$ret = trim(substr($line,strlen(self::gradeTag)));
			}
		}
		return $ret;
	}

	function proposedComment($text){
		$inComment=false;
		$ret = '';
		$nl = vpl_detect_newline($text);
		$lines = explode($nl,$text);
		foreach($lines as $line){
			$line = rtrim($line); //Remove \r, spaces & tabs
			$tline = trim($line);
			if($inComment) {
				if($tline == self::endCommentTag){
					$inComment=false;
				}else{
					$ret .= $line."\n";
				}
			}else{
				if(strpos($line,self::commentTag) === 0){
					$ret .= substr($line,strlen(self::commentTag))."\n";
				} elseif($tline == self::beginCommentTag){
					$inComment=true;
				}
			}
		}
		return $ret;
	}
	
	/**
	 * Add link to file line format filename:linenumber:
	 * @param text to be converted
	 * @return string text with links
	 */
	function add_filelink($text){
		//Format filename:linenumber
		$list = $this->get_submitted_fgm()->getFileList();
		if($list){
			foreach ($list as $filename){
				$wtext = $text;
				$text ='';
				$escapefilename=preg_quote($filename);
				$regexp = '/(.*?)('.$escapefilename.')\:( *)([0-9]+)(.*)/';
				while(preg_match($regexp,$wtext,$r)){
					$text .= $r[1].'<a href="#'.$r[2].'.'.$r[4].'">'.$r[2].':'.$r[3].$r[4].'</a>';
					$wtext =$r[5];
				}
				$text .= $wtext;
			}
		}
		return $text;
	}
	
	/**
	 * Convert compilation/execution result to HTML
	 * @param text to be converted
	 * @return string HTML
	 */
	function  result_to_HTML($text){
		if($text == ''){
			return '';
		}
		$html='';
		$nl = vpl_detect_newline($text);
		$lines = explode($nl,$text);
		$caseToShow='';
		foreach($lines as $line){
			$line = rtrim($line);
			$clean=ltrim($line);
			//End of case?
			if(strlen($caseToShow)>0 && !(strlen($clean)>0 && $clean[0]=='>')){
				$html .='<pre><i>';
				$html .= s($caseToShow);
				$html .= '</i></pre>';
				$caseToShow ='';
			}
			//Is title line
			if(strlen($line)>2 && $line[0]=='-') { //Title
				$line = substr($line,1);
				if($line[strlen($line)-1]==')') { //Has grade?
					$posOpen = strrpos($line,'(');
					if($posOpen !== false) {
						$grade = substr($line,$posOpen+1,strlen($line)-$posOpen-2);
						$grade = trim($grade);
						if($grade < 0){
							$html .= '<b>';
							$html .= s(substr($line,0,$posOpen));
							$html .= '</b><br />';
							//TODO implement grader information
							continue;
						}
					}
				}
				$html .= '<b>';
				$html .= s($line);
				$html .= '</b><br />';
			}elseif(strlen($clean)>0 && $clean[0]=='>') { //Case
				$pos=strpos($line,'>');
				$caseToShow .= substr($line,$pos+1)."\n";
			} elseif(strlen($clean)>8 && 
			         (substr($clean,0,5)=="http:" || substr($clean,0,6)=="https:")) {
				//Is url
				//output spaces
				$nspaces=strpos($line,'h');
				for($i=0; $i<$nspaces; $i++){
					$html .= '&nbsp;';
				}
				$spacepos = strpos($clean,' ');
				if($spacepos){
					$html .= '<a href="';
					$html .= urlencode(substr($clean,0,$spacepos));
					$html .= '">';
					$html .= s(substr($clean,$spacepos+1,strlen($clean)-$spacepos-1));
					$html .= '</a>';
				}else{
					$html .= '<a href="';
					$html .= urlencode($clean);
					$html .= '">';
					$html .= s($clean);
					$html .= '</a>';
				}
				$html .= '<br />';
			}else{ //Regular text
				$html .= $this->add_filelink(s($line)).'<br />';
			}
		}
		if(strlen($caseToShow)>0){
			$html .='<pre><i>';
			$html .= s($caseToShow);
			$html .= '</i></pre>';
		}
		return $html;
	}
	/**
	 * Add a new text to the list
	 */
	function  filter_feedback_add(&$list,$text,$grade=0){
		$text=trim($text);
		if(!isset($list[$text])){
			$list[$text]=new StdClass();
			$list[$text]->count=0;
			$list[$text]->grades=array();
		}
		$list[$text]->count++;
		$list[$text]->grades[$grade]=true;
	}
	/**
	 * Filter Convert compilation/execution result to HTML
	 * @param text to be filter
	 * @return array of mensajes
	 */
	function  filter_feedback(&$list){
		$text=$this->get_grade_comments();
		$nl = vpl_detect_newline($text);
		$lines = explode($nl,$text);
		foreach($lines as $line){
			$line = rtrim($line);
			$clean=ltrim($line);
			//Is title line
			if(strlen($line)>2 && $line[0]=='-') { //Title
				$line = substr($line,1);
				if($line[strlen($line)-1]==')') { //Has grade?
					$posOpen = strrpos($line,'(');
					if($posOpen !== false) {
						$grade = substr($line,$posOpen+1,strlen($line)-$posOpen-2);
						$grade = trim($grade);
						if($grade < 0){
							$this->filter_feedback_add($list,substr($line,0,$posOpen),$grade);
							continue;
						}
					}
				}
				$this->filter_feedback_add($list,$line,0);
			}
		}
	}

	/**
	 * Recopile execution data to be send to the jail
	 * @param $level=0 level of based on, maximum 10 levels
	 * @return object with files, limits, interactive and other info
	 */
	function prepare_execution($level=0, $vpl=null){
		global $CFG;
		if($level>10){
			print_error('Recursive basedon vpl definition');
		}
		if($vpl == null){
			$vpl = $this->vpl;
		}
		$vpl_instance = $vpl->get_instance();
		//Load basedon files if needed
		if($vpl_instance->basedon){
			$basedon = new mod_vpl(null,$vpl_instance->basedon);
			$data = $this->prepare_execution($level+1,$basedon);
		}else{
			$data=new stdClass();
			$data->files = array();
			$data->deletefiles = array();
			$data->maxtime = (int)$CFG->vpl_maxexetime;
			$data->maxfilesize = (int)$CFG->vpl_maxexefilesize;
			$data->maxmemory = (int)$CFG->vpl_maxexememory;
			$data->maxprocesses = (int)$CFG->vpl_maxexeprocesses;
		}
		//required files
		$sfg = $vpl->get_execution_fgm();
		$list = $sfg->getFileList();
		foreach ($list as $filename){
			if(isset($data->files[$filename]) &&
			($filename == 'vpl_run.sh' || $filename == 'vpl_debug.sh' || $filename == 'vpl_evaluate.sh')){
				$data->files[$filename].= "\n".$sfg->getFileData($filename);
			}else{
				$data->files[$filename] = $sfg->getFileData($filename);
			}
			$data->filestodelete[$filename] = 1;
		}
		$deletelist = $sfg->getFileKeepList();
		foreach ($deletelist as $filename){
			unset($data->filestodelete[$filename]);
		}
	
		if($vpl_instance->maxexetime && $vpl_instance->maxexetime<$data->maxtime){
			$data->maxtime=(int)$vpl_instance->maxexetime;
		}
		if($vpl_instance->maxexememory && $vpl_instance->maxexememory<$data->maxmemory){
			$data->maxmemory=(int)$vpl_instance->maxexememory;
		}
		if($vpl_instance->maxexefilesize && $vpl_instance->maxexefilesize<$data->maxfilesize){
			$data->maxfilesize=(int)$vpl_instance->maxexefilesize;
		}
		if($vpl_instance->maxexeprocesses && $vpl_instance->maxexeprocesses<$data->maxprocesses){
			$data->maxprocesses=(int)$vpl_instance->maxexeprocesses;
		}
		if($level >0 ){
			return $data;
		}
		//Submited files
		$sfg = $this->get_submitted_fgm();
		$list = $sfg->getFileList();
		foreach ($list as $filename){
			if(!isset($data->files[$filename])){
				$data->files[$filename] = $sfg->getFileData($filename);
			}
		}
		//Info send with script
		$info ="#!/bin/bash\n";
		$info .='export LANG='.vpl_get_lang(true)."\n";
		$info .='export VPL_MAXTIME='.$data->maxtime."\n";
		$info .='export VPL_FILEBASEURL='.$CFG->wwwroot.'file.php/';
		$info .= $vpl->get_course_module()->id."/\n";
		$num=0;
		foreach ($list as $filename){
			$info .='export VPL_SUBFILE'.$num.'='.$filename."\n";
			$num++;
		}
		//TODO add options info
		foreach(array('vpl_run.sh','vpl_debug.sh','vpl_evaluate.sh') as $script){
			if(isset($data->files[$script])){
				$data->files[$script]="#!/bin/bash\n".$data->files[$script];
			}
		}
		//Add script file with VPL enviroment information
		$data->files['vpl_enviroment.sh']=$info;
		return $data;
	}

	const compilationFN='compilation.txt';
	const executionFN='execution.txt';
	function saveCE($result){
		$compfn = $this->get_data_directory().'/'.self::compilationFN;
		if(file_exists($compfn)){
			unlink($compfn);
		}
		$execfn = $this->get_data_directory().'/'.self::executionFN;
		if(file_exists($execfn)){
			unlink($execfn);
		}
		file_put_contents($compfn,$result['compilation']);
		if($result['executed']>0){
			file_put_contents($execfn,$result['execution']);
		}
	}

	function getCE(){
		$ret = array();
		$compfn = $this->get_data_directory().'/'.self::compilationFN;
		if(file_exists($compfn)){
			$ret['compilation']=file_get_contents($compfn);
		}else{
			$ret['compilation']=0;
		}
		$execfn = $this->get_data_directory().'/'.self::executionFN;
		if(file_exists($execfn)){
			$ret['executed']=1;
			$ret['execution']=file_get_contents($execfn);
		}else{
			$ret['executed']=0;
		}
		return $ret;
	}
	
	function send_CE_to_editor($response,$window='window.parent',$defer=false){
		echo '<div style="display: none">';
		if($response['compilation']){
			$rescompilation=$this->result_to_HTML($response['compilation']);
			if(strlen($rescompilation)){
				echo '<div id="compilation"><h3>'.get_string('compilation',VPL).'</h3>';
				echo $rescompilation.'</div>';
				echo "\n";
			}
		}
		if($response['executed']>0){
			$raw_execution = $response['execution'];
			$proposed_comments = $this->proposedComment($raw_execution);
			$proposed_grade = $this->proposedGrade($raw_execution);
			$resexecution=$this->result_to_HTML($proposed_comments);
			if(strlen($resexecution)){
				echo '<div id="execution"><h3>'.get_string('comments',VPL).'</h3>';
				echo $resexecution.'</div>';
				echo "\n";
			}
			if(strlen($proposed_grade)){
				echo '<div id="grade">'.get_string('proposedgrade',VPL,$proposed_grade).'</div>';
				echo "\n";
			}
		}
		echo vpl_include_js('transferResult('.$window.');',$defer);
		echo "\n";
		echo '</div>';
		@ob_flush();
		flush();
	}
	function get_detail(){
		$ret='';
		$subf = $this->get_submitted_fgm();
		$filelist = $subf->getFileList();
		foreach($filelist as $filename){
			$data = $subf->getFileData($filename);
			if($ret>''){
				$ret.=', ';
			}
			//TODO too slow calculus
			$nl = vpl_detect_newline($data);
			$ret .= $filename.' '.strlen($data).'b '.count(explode($nl,$data)).'l';
		}
		return $ret;
	}
	function get_CE_parms(){
		$response = $this->getCE();
		$params = '';
		if($response['compilation']){
			$rescompilation=$this->result_to_HTML($response['compilation']);
			if(strlen($rescompilation)){
				$data = '<h3>'.get_string('compilation',VPL).'</h3>';
				$data .= $rescompilation;
				$params .= vpl_param_tag('compilation',$data);
			}
		}
		if($response['executed']>0){
			$raw_execution = $response['execution'];
			$proposed_comments = $this->proposedComment($raw_execution);
			$proposed_grade = $this->proposedGrade($raw_execution);
			$resexecution=$this->result_to_HTML($proposed_comments);
			if(strlen($resexecution)){
				$data = '<h3>'.get_string('comments',VPL).'</h3>';
				$data .= $resexecution;
				$params .= vpl_param_tag('evaluation',$data);
			}
			if(strlen($proposed_grade)){
				$params .= vpl_param_tag('grade',$proposed_grade);
			}
		}
		return $params;
	}
}
?>